package com.xydl.cac.iec;

import com.xydl.cac.entity.IcdFile;
import com.xydl.cac.entity.constants.Constants;
import com.xydl.cac.exception.BusinessException;
import com.xydl.cac.repository.IcdFileRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;

@Service
@Slf4j
public class IecServerService {

    @Resource
    IcdFileRepository fileRepository;

    IecServer iecServer = new IecServer();

    @PostConstruct
    private void init() {
        List<IcdFile> list = fileRepository.findBySrvAndStart(Constants.Server, Constants.TRUE);
        if (!CollectionUtils.isEmpty(list)) {
            IcdFile icdFile = list.get(0);
            try {
                this.startServer(icdFile.getId());
            } catch (Exception ignore) {
            }
        }
    }

    public void startServer(Integer fileId) throws BusinessException {
        Optional<IcdFile> optional = fileRepository.findById(fileId);
        if (!optional.isPresent()) {
            throw new BusinessException("未找到该文件");
        }
        IcdFile icdFile = optional.get();
        if (icdFile.getSrv() != Constants.Server) {
            throw new BusinessException("该文件不是服务端文件");
        }
        try {
            iecServer.start(icdFile.getXml(), 102);
            iecServer.filename = icdFile.getFilename();
            iecServer.fileId = icdFile.getId();
            RealTimeDataService.iecServer = iecServer;
            icdFile.setStart(Constants.TRUE);
            fileRepository.save(icdFile);
        } catch (Exception e) {
            log.error("启动IEC61850服务端异常.", e);
        }
    }

    public void stopServer() {
        this.stop();
        if (iecServer.fileId != null) {
            Optional<IcdFile> optional = fileRepository.findById(iecServer.fileId);
            if (optional.isPresent()) {
                IcdFile icdFile = optional.get();
                icdFile.setStart(Constants.FALSE);
                fileRepository.save(icdFile);
            }
        }
    }

    @PreDestroy
    private void stop() {
        log.info("关闭IecServer服务.");
        RealTimeDataService.iecServer = null;
        iecServer.close();
    }

    public HashMap<String, Object> status() {
        HashMap<String, Object> map = new HashMap<>();
        map.put("port", 102);
        map.put("started", iecServer.started);
        map.put("filename", iecServer.filename);
        map.put("fileId", iecServer.fileId);
        return map;
    }
}
